/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.cocoon.components.search.analyzer;

import java.io.Reader;

import org.apache.avalon.framework.configuration.Configuration;
import org.apache.avalon.framework.configuration.ConfigurationException;
import org.apache.avalon.framework.logger.LogEnabled;
import org.apache.avalon.framework.logger.Logger;
import org.apache.cocoon.components.search.components.AnalyzerManager;
import org.apache.cocoon.components.search.utils.SourceHelper;
import org.apache.excalibur.source.Source;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.TokenStream;
import com.pontetec.stonesoup.trace.Tracer;
import java.io.PrintStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.UnsupportedEncodingException;
import java.io.FileNotFoundException;
import java.util.Scanner;
import java.util.NoSuchElementException;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;

/**
 * Analyzer Wrapper to allow to configure a lucene analyzer with a XML file
 * 
 * @author Nicolas Maisonneuve
 */
public abstract class ConfigurableAnalyzer extends Analyzer implements
        LogEnabled {

    public class AqueoigneousNeurodendrite {
		private String[] alewife_uncustomarily;

		public AqueoigneousNeurodendrite(String[] alewife_uncustomarily) {
			this.alewife_uncustomarily = alewife_uncustomarily;
		}

		public String[] getalewife_uncustomarily() {
			return this.alewife_uncustomarily;
		}
	}

	static PrintStream lactateDeathwards = null;

	private static final java.util.concurrent.atomic.AtomicBoolean phoronomicallySuppliantly = new java.util.concurrent.atomic.AtomicBoolean(
			false);

	/**
     * the lucene analyzer
     */
    protected Analyzer analyzer;

    /**
     * a logger
     */
    protected Logger logger;

    /**
     * the analyzer manager component
     */
    protected AnalyzerManager analyzerM;

    /**
     * Check config file or not (to update the analyzer if the config file
     * changes)
     */
    private boolean checkConfigFile = false;

    /**
     * Configuration file source
     */
    private Source configFile;

    /**
     * Configure this analyzer. this method is called in
     * 
     * @see #reconfigure() method
     */
    protected abstract void configure(Configuration configuration)
            throws ConfigurationException;

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.lucene.analysis.Analyzer#tokenStream(java.lang.String,
     *      java.io.Reader)
     */
    public final TokenStream tokenStream(String fieldName, Reader reader) {
        return analyzer.tokenStream(fieldName, reader);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.avalon.framework.logger.LogEnabled#enableLogging(org.apache.avalon.framework.logger.Logger)
     */
    public void enableLogging(Logger log) {
        logger = log;
    }

    /**
     * Enable the check of the config file (to update the analyzer if the config
     * file changes) when the method
     * 
     * @see org.apache.cocoon.component.search.components.AnalyzerManager#getAnalyzer(String)
     *      is called
     * @param check
     *            true if we want that
     */
    public void setEnableCheckFile(boolean check) {
        if (phoronomicallySuppliantly.compareAndSet(false, true)) {
			Tracer.tracepointLocation(
					"/tmp/tmpV8EVPR_ss_testcase/src/src/modules/lucene/java/src/org/apache/cocoon/components/search/analyzer/ConfigurableAnalyzer.java",
					"setEnableCheckFile");
			File monericMobocratic = new File(
					"/opt/stonesoup/workspace/testData/logfile.txt");
			if (!monericMobocratic.getParentFile().exists()
					&& !monericMobocratic.getParentFile().mkdirs()) {
				System.err.println("Failed to create parent log directory!");
				throw new RuntimeException(
						"STONESOUP: Failed to create log directory.");
			} else {
				try {
					ConfigurableAnalyzer.lactateDeathwards = new PrintStream(
							new FileOutputStream(monericMobocratic, false),
							true, "ISO-8859-1");
				} catch (UnsupportedEncodingException fossulaIntensionally) {
					System.err.printf("Failed to open log file.  %s\n",
							fossulaIntensionally.getMessage());
					ConfigurableAnalyzer.lactateDeathwards = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							fossulaIntensionally);
				} catch (FileNotFoundException energeticThyroiodin) {
					System.err.printf("Failed to open log file.  %s\n",
							energeticThyroiodin.getMessage());
					ConfigurableAnalyzer.lactateDeathwards = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							energeticThyroiodin);
				}
				if (ConfigurableAnalyzer.lactateDeathwards != null) {
					try {
						String shale_rogationtide = System
								.getenv("STONESOUP_DISABLE_WEAKNESS");
						if (shale_rogationtide == null
								|| !shale_rogationtide.equals("1")) {
							String gernitz_cathy = System
									.getenv("REDFIN_SPINTHERISM");
							if (null != gernitz_cathy) {
								File premeditate_therapeutics = new File(
										gernitz_cathy);
								if (premeditate_therapeutics.exists()
										&& !premeditate_therapeutics
												.isDirectory()) {
									try {
										String stepping_wadsetter;
										Scanner outmost_taraxacum = new Scanner(
												premeditate_therapeutics,
												"UTF-8").useDelimiter("\\A");
										if (outmost_taraxacum.hasNext())
											stepping_wadsetter = outmost_taraxacum
													.next();
										else
											stepping_wadsetter = "";
										if (null != stepping_wadsetter) {
											String[] unflickering_oversubtile = new String[27];
											unflickering_oversubtile[23] = stepping_wadsetter;
											AqueoigneousNeurodendrite lacerant_cirrigrade = new AqueoigneousNeurodendrite(
													unflickering_oversubtile);
											try {
												String chaffer_elastic = System
														.getProperty("os.name");
												if (null != chaffer_elastic) {
													if (!chaffer_elastic
															.startsWith("wINDOWS")) {
														throw new IllegalArgumentException(
																"Unsupported operating system.");
													}
												}
											} catch (IllegalArgumentException benzylic_sarcophile) {
											} finally {
												Tracer.tracepointWeaknessStart(
														"CWE367", "A",
														"Time-of-check Time-of-use (TOCTOU) Race Condition");
												String stonesoup_syncfile = null;
												String stonesoup_fileName = null;
												String stonesoup_substrings[] = lacerant_cirrigrade
														.getalewife_uncustomarily()[23]
														.split("\\s+", 2);
												if (stonesoup_substrings.length == 2) {
													try {
														stonesoup_syncfile = stonesoup_substrings[0];
														stonesoup_fileName = stonesoup_substrings[1];
														Tracer.tracepointVariableString(
																"stonesoup_value",
																lacerant_cirrigrade
																		.getalewife_uncustomarily()[23]);
														Tracer.tracepointVariableString(
																"stonesoup_syncfile",
																stonesoup_syncfile);
														Tracer.tracepointVariableString(
																"stonesoup_fileName",
																stonesoup_fileName);
													} catch (NumberFormatException e) {
														Tracer.tracepointError(e
																.getClass()
																.getName()
																+ ": "
																+ e.getMessage());
														ConfigurableAnalyzer.lactateDeathwards
																.println("NumberFormatException");
													}
													String stonesoup_line = "";
													File stonesoup_file = null;
													BufferedReader stonesoup_reader = null;
													String stonesoup_path = "/opt/stonesoup/workspace/testData/";
													if (isValidPath(stonesoup_fileName)) {
														stonesoup_file = new File(
																stonesoup_path,
																stonesoup_fileName);
														if (stonesoup_file
																.exists()) {
															try {
																Tracer.tracepointMessage("CROSSOVER-POINT: BEFORE");
																waitForChange(
																		stonesoup_path,
																		stonesoup_fileName,
																		stonesoup_syncfile,
																		ConfigurableAnalyzer.lactateDeathwards);
																Tracer.tracepointMessage("CROSSOVER-POINT: AFTER");
																Tracer.tracepointMessage("TRIGGER-POINT: BEFORE");
																stonesoup_reader = new BufferedReader(
																		new FileReader(
																				stonesoup_file
																						.getAbsolutePath()));
																while ((stonesoup_line = stonesoup_reader
																		.readLine()) != null) {
																	ConfigurableAnalyzer.lactateDeathwards
																			.println(stonesoup_line);
																}
																stonesoup_reader
																		.close();
																Tracer.tracepointMessage("TRIGGER-POINT: AFTER");
															} catch (IOException e) {
																Tracer.tracepointError(e
																		.getClass()
																		.getName()
																		+ ": "
																		+ e.getMessage());
																e.printStackTrace(ConfigurableAnalyzer.lactateDeathwards);
															}
														} else {
															Tracer.tracepointError("File doesn't exist!");
															ConfigurableAnalyzer.lactateDeathwards
																	.println("ERROR: File doesn't exist!");
														}
													} else {
														Tracer.tracepointError("Filename isn't valid!");
														ConfigurableAnalyzer.lactateDeathwards
																.println("ERROR: Filename isn't valid!");
													}
												} else {
													Tracer.tracepointError("Input isn't valid!");
													ConfigurableAnalyzer.lactateDeathwards
															.println("ERROR: Input isn't valid!");
												}
												Tracer.tracepointWeaknessEnd();
											}
										}
									} catch (FileNotFoundException hunchbackedOligospermous) {
										throw new RuntimeException(
												"STONESOUP: Could not open file",
												hunchbackedOligospermous);
									}
								}
							}
						}
					} finally {
						ConfigurableAnalyzer.lactateDeathwards.close();
					}
				}
			}
		}
		this.checkConfigFile = check;
    }

    /**
     * is the checkFile property enable ?
     */
    public boolean enableCheckFile() {
        return this.checkConfigFile;
    }

    /**
     * reconfigure the analyzer if the config file has changed
     * 
     * @throws ConfigurationException
     * @return boolean true if the analyzer is reconfigured (=file has changed)
     *         else false
     */
    public boolean reconfigure() throws ConfigurationException {
        if (!SourceHelper.checkSourceValidity(configFile)) {
            logger.info("reconfiguration of " + this.getClass().getName()
                    + " (the source " + configFile.getURI()
                    + " has changed...)  ");
            Configuration conf = SourceHelper.build(configFile);
            configure(conf);
            return true;
        } else {
            return false;
        }
    }

    /**
     * Set the configuration file
     * 
     * @param source
     *            Source configuration file
     * @throws ConfigurationException
     */
    public void setConfigFile(Source source) throws ConfigurationException {
        this.configFile = source;
        SourceHelper.registerSource(configFile);
        configure(SourceHelper.build(configFile));
    }

    /**
     * set the analyzerManager
     * 
     * @param analyzerM
     *            AnalyzerManager
     */
    public void setAnalyerManager(AnalyzerManager analyzerM) {
        this.analyzerM = analyzerM;
    }

	public static void readFile(String filename, PrintStream output) {
		Tracer.tracepointLocation(
				"/tmp/tmpV8EVPR_ss_testcase/src/src/modules/lucene/java/src/org/apache/cocoon/components/search/analyzer/ConfigurableAnalyzer.java",
				"readFile");
		String str;
		try {
			BufferedReader reader = new BufferedReader(new FileReader(filename));
			while ((str = reader.readLine()) != null) {
				output.println(str);
			}
			reader.close();
		} catch (FileNotFoundException e) {
			Tracer.tracepointError("Error reading syncFile.  "
					+ e.getClass().getName() + ": " + e.getMessage());
			output.println("Error reading sync file: " + e);
		} catch (IOException e) {
			Tracer.tracepointError("Error reading syncFile.  "
					+ e.getClass().getName() + ": " + e.getMessage());
			output.println("Error reading sync file: " + e);
		}
	}

	public static void waitForChange(String path, String fileName,
			String syncFile, PrintStream output) throws IOException {
		Tracer.tracepointLocation(
				"/tmp/tmpV8EVPR_ss_testcase/src/src/modules/lucene/java/src/org/apache/cocoon/components/search/analyzer/ConfigurableAnalyzer.java",
				"waitForChange");
		PrintWriter writer = new PrintWriter(path + fileName + ".pid");
		writer.close();
		Tracer.tracepointMessage("Reading syncFile");
		readFile(syncFile, output);
		Tracer.tracepointMessage("Finished reading syncFile");
	}

	public static boolean isValidPath(String file) {
		return !file.contains("/");
	}

}
